import processing.core.*; 
import processing.xml.*; 

import processing.opengl.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class SpaceBoxes extends PApplet {

/**
 * SpaceBoxes
 * Like the Windows starfield screen saver, but boxier
 * Looks pretty cool in fullscreen: Set FULLSCREEN=true and start with "Present" (Ctrl-Shift-R)
 *
 * Joerg Reuter, jore[at]stachelig[dot]de
 * Creative Commons Attribution 3.0 license
 * http://stachelig.de
 */



boolean FULLSCREEN = true;

int MAX_X = FULLSCREEN ? screen.width : 1280;
int MAX_Y = FULLSCREEN ? screen.height : 720;

float FAR_Z = -4000.0f;
float NEAR_Z = 1000;

int SIZE = 80;
int BLOCK_NUM = 600;

Block[] blocks = new Block[BLOCK_NUM];

public void setup() {
  size(MAX_X, MAX_Y, OPENGL);
  
  camera(0, 0, -90, 0, 0, -100, 0, 1, 0);
  float cameraZ = ((width/2.0f) / tan(PI*60.0f/360.0f));
  // adjusted from perspective() reference to narrow fov
  perspective(PI/4.0f, PApplet.parseFloat(width)/height, cameraZ/10.0f, cameraZ*10.0f);
  
  noStroke();
  shininess(.0010f);

  // create boxes
  for(int i=0; i<BLOCK_NUM; i++)
    blocks[i] = new Block();
}

public void draw() {
  background(0x000000);
  directionalLight(220, 220, 180, 1, 1, -1);
  directionalLight(50, 80, 120, -1, -1, -1);
  
  // draw boxes
  for(int i=0; i<BLOCK_NUM; i++)
    blocks[i].draw();
}

// class to represent a flying box
class Block {
  float x, y, z;
  float rotX, rotY, rotZ;
  float speed, size;
  int t; // time, used for movement & rotation
  
  Block() {
    rebirth();
    z = random(FAR_Z, NEAR_Z);
  }
  
  public void draw() {
      z+=speed;
      if(z>NEAR_Z) {
        rebirth();
      } else {
        pushMatrix();
        translate(x, y, z);
        rotateX(t*rotX);
        rotateY(t*rotY);
        rotateZ(t*rotZ);
        
        // far away boxes fade out
        fill(constrain(abs(FAR_Z-z)/10.0f, 0, 255), 255);
        
        box(size, size, size);
        popMatrix();
        t++;
      }
  }
  
  public void rebirth() {
    float dist = random(SIZE+SIZE/2, abs(FAR_Z)/3);
    float deg = random(PI*2);
    x = sin(deg)*dist;
    y = cos(deg)*dist;
    z = FAR_Z;
    t = 0;
    rotX=random(.025f);
    rotY=random(.025f);
    rotZ=random(.025f);
    speed=random(8.0f, 30.0f);
    size=random(SIZE/4, SIZE);
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--present", "--bgcolor=#666666", "--hide-stop", "SpaceBoxes" });
  }
}
